import int32Mul from './lib/int32Mul';
import updateKeys from './lib/updateKeys';

export default function getDecryptor(decryptionKeys: [number, number, number]) {
  let [key0, key1, key2] = decryptionKeys;

  return (encryptedChunk: Buffer) => {
    const decryptedChunk = Buffer.alloc(encryptedChunk.length);

    for (let i = 0; i < encryptedChunk.length; i += 1) {
      //read byte
      let curChar = encryptedChunk.readUInt8(i);

      //decrypt byte
      const keyPart = ((key2 | 2) >>> 0) & 0xFFFF;
      const decryptedByte = int32Mul(keyPart, keyPart ^ 1) >>> 8;
      curChar = (curChar ^ (decryptedByte & 0xFF)) & 0xFF;

      //update keys
      [key0, key1, key2] = updateKeys([key0, key1, key2], curChar);

      //write byte
      decryptedChunk.writeUInt8(curChar, i);
    }

    return decryptedChunk;
  };
}
